@@ -1,12 +1,17 @@
 import argparse
+import cProfile
 import os
-from argparse import ArgumentParser, Namespace
+import pstats
+from argparse import Namespace, ArgumentParser
+from contextlib import contextmanager
 from pathlib import Path
+from pstats import SortKey
 from typing import List, Optional, Dict
 
 import sys
 
 from pkm.api.dependencies.dependency import Dependency
+from pkm.api.packages.package_installation_info import StoreMode
 from pkm.api.environments.environment import Environment
 from pkm.api.environments.environments_zoo import EnvironmentsZoo
 from pkm.api.packages.package_installation import PackageInstallationTarget
@@ -14,28 +19,91 @@ from pkm.api.pkm import pkm, HasAttachedRepository
 from pkm.api.projects.project import Project
 from pkm.api.projects.project_group import ProjectGroup
 from pkm.api.repositories.repositories_configuration import RepositoryInstanceConfig
+from pkm.api.versions.version_specifiers import VersionSpecifier
 from pkm.utils.commons import UnsupportedOperationException
+from pkm.utils.enums import enum_by_name
 from pkm.utils.processes import execvpe
-from pkm.utils.resources import ResourcePath
 from pkm_cli import cli_monitors
-from pkm_cli.context import Context
+from pkm_cli.api.tasks.tasks_runner import TasksRunner
+from pkm_cli.api.templates.template_runner import TemplateRunner
+from pkm_cli.controllers.env_controller import EnvController
+from pkm_cli.controllers.prj_controller import PrjController
+from pkm_cli.controllers.self_controller import SelfController
 from pkm_cli.display.display import Display
 from pkm_cli.reports.added_repositories_report import AddedRepositoriesReport
 from pkm_cli.reports.environment_report import EnvironmentReport
 from pkm_cli.reports.installed_repositories_report import InstalledRepositoriesReport
 from pkm_cli.reports.package_report import PackageReport
+from pkm_cli.reports.pkm_report import PkmReport
 from pkm_cli.reports.project_report import ProjectReport
-from pkm_cli.scaffold.engine import ScaffoldingEngine
+from pkm_cli.reports.report import Report
 from pkm_cli.utils.clis import command, Arg, create_args_parser, Command, with_extras
+from pkm_cli.utils.context import Context
+
+context: Optional[Context] = None
+tasks: Optional[TasksRunner] = None
 
 
 def _cli_container() -> PackageInstallationTarget:
     global_env = Environment.current()
-    if pkm_container := global_env.app_containers.container_of('pkm-cli'):
+    if pkm_container := global_env.package_containers.container_of('pkm-cli'):
         return pkm_container.installation_target
     return global_env.installation_target
 
 
+def status_command(path: str):
+    return command(
+        path, Arg(['-l', '--list-options'], action='store_true'), Arg(['-i', '--include'], nargs='*'),
+        Arg(['-x', '--exclude'], nargs='*'))
+
+
+def _show_status(report: Report, args: Namespace):
+    Display.print()
+
+    if args.list_options:
+        report.display_options()
+        return
+
+    options = {}
+    options.update({s: True for s in args.include or []})
+    options.update({s: False for s in args.exclude or []})
+
+    report.display(options)
+
+
+# noinspection PyUnusedLocal
+@status_command('pkm self status')
+def self_version(args: Namespace):
+    _show_status(PkmReport(), args)
+
+
+@command('pkm self update', Arg(["-s", "--version-spec"], default="*"))
+def self_update(args: Namespace):
+    SelfController().update(VersionSpecifier.parse(args.version_spec))
+
+
+@command(
+    'pkm self install',
+    Arg(["-u", "--update"], action='store_true', help="update the given packages if already installed"),
+    Arg(["-m", "--mode"], required=False, default='editable', choices=['editable', 'copy'],
+        help="choose the installation mode for the requested packages."),
+    Arg('packages', nargs=argparse.REMAINDER, help="the packages to install (support pep508 dependency syntax)"))
+def self_install(args: Namespace):
+    dependencies = [Dependency.parse(d) for d in args.packages]
+    store_mode = enum_by_name(StoreMode, args.mode.upper())
+    SelfController().install_plugins(dependencies, store_mode=store_mode, update=args.update)
+
+
+@command(
+    'pkm self uninstall',
+    Arg(["-f", "--force"], action="store_true",
+        help="remove the requested packages even if they are dependant of other packages, "
+             "will not remove any other packages or update pyproject"),
+    Arg('package_names', nargs=argparse.REMAINDER, help="the packages to remove"))
+def self_uninstall(args: Namespace):
+    SelfController().uninstall_plugin(args.package_names, force=args.force)
+
+
 @command('pkm repos install', Arg(["-u", "--update"], action="store_true"), Arg('names', nargs=argparse.REMAINDER))
 def install_repos(args: Namespace):
     if not args.global_context:
@@ -60,16 +128,16 @@ def show_installed_repositories(_: Namespace):
 
 @command(
     'pkm repos add',
-    Arg("name"),
+    Arg("repo_name"),
     Arg("type", action=with_extras()),
     Arg(['-b', '--bind-only'], action="store_true", required=False))
 def add_repo(args: Namespace):
     def add(with_repo: HasAttachedRepository):
         with_repo.repository_management.add_repository(
-            args.name, args.type, getattr(args, 'type_extras', {}), args.bind_only)
+            args.repo_name, args.type, getattr(args, 'type_extras', {}), args.bind_only)
 
     on_environment, on_project, on_project_group, = add, add, add
-    Context.of(args).run(**locals())
+    context.run(**locals())
 
 
 @command('pkm repos remove', Arg("name"))
@@ -78,7 +146,7 @@ def remove_repo(args: Namespace):
         with_repo.repository_management.remove_repository(args.name)
 
     on_environment, on_project, on_project_group, = rm, rm, rm
-    Context.of(args).run(**locals())
+    context.run(**locals())
 
 
 @command('pkm repos show added')
@@ -87,7 +155,7 @@ def show_added_repositories(args: Namespace):
         AddedRepositoriesReport(with_repo).display()
 
     on_environment, on_project, on_project_group, = show, show, show
-    Context.of(args).run(**locals())
+    context.run(**locals())
 
 
 @command('pkm run', Arg('cmd', nargs=argparse.REMAINDER))
@@ -100,29 +168,17 @@ def run(args: Namespace):
             sys.exit(execvpe(args.cmd[0], args.cmd[1:], os.environ))
 
     def on_project(project: Project):
-        on_environment(project.attached_environment)
+        if args.cmd[0].startswith("@"):
+            task_name = args.cmd[0][1:]
+            task_args = args.cmd[1:]
+            if '-h' in task_args:
+                Display.print(tasks.describe(task_name))
+            else:
+                sys.exit(tasks.run(task_name, task_args))
+        else:
+            on_environment(project.attached_environment)
 
-    Context.of(args).run(**locals())
-
-
-# noinspection PyUnusedLocal
-@command('pkm shell', Arg(["-e", '--execute'], required=False, nargs=argparse.REMAINDER))
-def shell(args: Namespace):
-    def on_environment(env: Environment):
-        with env.activate():
-            if execution := args.execute:
-                sys.exit(execvpe(execution[0], execution[1:], os.environ))
-
-            import xonsh.main
-            sys.exit(xonsh.main.main([]))
-
-    def on_project(project: Project):
-        on_environment(project.attached_environment)
-
-    def on_free_context():
-        on_environment(Environment.current())
-
-    Context.of(args).run(**locals())
+    context.run(**locals())
 
 
 # noinspection PyUnusedLocal
@@ -138,7 +194,7 @@ def build(args: Namespace):
     def on_project_group(project_group: ProjectGroup):
         project_group.build_all()
 
-    Context.of(args).run(**locals())
+    context.run(**locals())
 
 
 @command(
@@ -149,7 +205,7 @@ def vbump(args: Namespace):
         new_version = project.bump_version(args.particle)
         Display.print(f"Version bumped to: {new_version}")
 
-    Context.of(args).run(**locals())
+    context.run(**locals())
 
 
 @command(
@@ -158,7 +214,7 @@ def vbump(args: Namespace):
     Arg(["-f", "--force"], action='store_true', help="forcefully remove and reinstall the given pacakges"),
     Arg(["-a", "--app"], action='store_true', help="install package in containerized application mode"),
     Arg(["-u", "--update"], action='store_true', help="update the given packages if already installed"),
-    Arg(["-m", "--mode"], required=False, default='editable', choices=['editable', 'copy'],
+    Arg(["-m", "--mode"], required=False, default='auto', choices=['editable', 'copy', 'auto'],
         help="choose the installation mode for the requested packages."),
     Arg(['-s', '--site'], required=False, choices=['user', 'system'],
         help="applicable for global-context, which site to use - defaults to 'user'"),
@@ -173,7 +229,7 @@ def install(args: Namespace):
     """
 
     dependencies = [Dependency.parse(it) for it in args.packages]
-    editable = args.mode == 'editable'
+    store_mode = enum_by_name(StoreMode, args.mode.upper())
 
     def register_repo_bindings(contex: HasAttachedRepository):
         if repo := args.repo:
@@ -183,8 +239,7 @@ def install(args: Namespace):
                 contex.repository_management.register_bindings([d.package_name for d in dependencies], repo)
                 args.force = True
         elif repo_type := args.unnamed_repo:
-            print("DBG: recognized unnamed repo option")
-            instance_config = RepositoryInstanceConfig(repo_type, None, None, getattr(args, 'unnamed_repo_extras', {}))
+            instance_config = RepositoryInstanceConfig(repo_type, True, getattr(args, 'unnamed_repo_extras', {}))
             contex.repository_management.register_bindings([d.package_name for d in dependencies], instance_config)
             args.force = True
 
@@ -199,7 +254,12 @@ def install(args: Namespace):
             raise UnsupportedOperationException("application install as project dependency is not supported")
 
         force(project.attached_environment.installation_target)
-        project.dev_install(dependencies, optional_group=args.optional, update=args.update, editable=editable)
+        ctrl = PrjController(project)
+        if dependencies:
+            ctrl.install_dependencies(
+                dependencies, optional_group=args.optional, update=args.update, store_mode=store_mode)
+        else:
+            ctrl.install_project(optional_group=args.optional)
 
     def on_environment(env: Environment):
         nonlocal dependencies
@@ -210,17 +270,31 @@ def install(args: Namespace):
         if dependencies:
             target = env.installation_target
             if args.app:
-                target = env.app_containers.install(
-                    dependencies[0], editable=editable, update=args.update and len(dependencies) == 1)
+                target = env.package_containers.install(
+                    dependencies[0], store_mode=store_mode,
+                    update=args.update and len(dependencies) == 1).installation_target
                 dependencies = dependencies[1:]
 
             if dependencies:
                 force(target)
                 updates = [d.package_name for d in dependencies] if args.update else None
-                editables = {d.package_name: editable for d in dependencies}
-                target.install(dependencies, updates=updates, editables=editables)
+                store_modes = {d.package_name: store_mode for d in dependencies}
+                target.install(dependencies, updates=updates, store_mode=store_modes)
 
-    Context.of(args).run(**locals())
+    context.run(**locals())
+
+
+@command(
+    "pkm uninstall-orphans",
+    Arg(["-a", "--app"], action='store_true', help="uninstall orphans in containerized package"))
+def uninstall_orphans(args: Namespace):
+    def on_environment(env: Environment):
+        EnvController(env).uninstall_orphans(args.app)
+
+    def on_project(prj: Project):
+        on_environment(prj.attached_environment)
+
+    context.run(**locals())
 
 
 @command('pkm uninstall', Arg(["-a", "--app"], action='store_true', help="remove containerized packages"),
@@ -228,43 +302,25 @@ def install(args: Namespace):
              help="remove the requested packages even if they are dependant of other packages, "
                   "will not remove any other packages or update pyproject"),
          Arg('package_names', nargs=argparse.REMAINDER, help="the packages to remove"))
-def remove(args: Namespace):
+def uninstall(args: Namespace):
     """
     remove packages from the current context
     """
 
-    if not (package_names := args.package_names):
+    if not args.package_names:
         raise ValueError("no package names are provided to be removed")
 
-    app_install = bool(args.app)
-
-    def _remove(target: PackageInstallationTarget, packages: List[str] = package_names):
-        if args.force:
-            for package in packages:
-                target.force_remove(package)
-        else:
-            target.uninstall(packages)
-
     def on_project(project: Project):
-        if app_install:
+        if args.app:
             raise UnsupportedOperationException("application install/remove as project dependency is not supported")
-
-        if args.force:
-            _remove(project.attached_environment.installation_target)
-        else:
-            project.dev_remove(package_names)
+        PrjController(project).uninstall_dependencies(args.package_names, args.force)
 
     def on_environment(env: Environment):
-        if app_install:
-            if container := env.installation_target.app_containers.container_of(package_names[0]):
-                if len(package_names) == 1:
-                    container.uninstall()
-                else:
-                    _remove(container.installation_target, package_names[1:])
-        else:
-            _remove(env.installation_target, package_names)
+        app = args.package_names[0] if args.app else None
+        package_names = args.package_names[1:] if args.app else args.package_names
+        EnvController(env).uninstall(package_names, args.force, app)
 
-    Context.of(args).run(**locals())
+    context.run(**locals())
 
 
 @command('pkm publish', Arg('repo', action=with_extras()), Arg(['-s', '--save'], action='store_true'))
@@ -295,24 +351,31 @@ def publish(args: Namespace):
         if args.save and publish_auth:
             publish_auth.add_auth_args(args.repo, auth_args)
 
-    Context.of(args).run(**locals())
+    context.run(**locals())
 
 
-@command('pkm new', Arg('template'), Arg('template_args', nargs=argparse.REMAINDER))
+@command(
+    'pkm new',
+    Arg('template'), Arg(["-o", "--overwrite"], action="store_true"),
+    Arg('template_args', nargs=argparse.REMAINDER))
 def new(args: Namespace):
-    ScaffoldingEngine().render(
-        ResourcePath('pkm_cli.scaffold', f"new_{args.template}.tar.gz"), Path.cwd(), args.template_args)
+    tr = TemplateRunner()
+    if "-h" in args.template_args:
+        Display.print(tr.describe(args.template))
+    else:
+        tr.run(args.template, Path.cwd(), args.template_args, bool(args.overwrite))
+        Display.print("Template Execution Completed Successfully.")
 
 
-@command('pkm show context')
-def show_context(args: Namespace):
+@status_command('pkm status')
+def status(args: Namespace):
     def on_project(project: Project):
-        ProjectReport(project).display()
+        _show_status(ProjectReport(project), args)
 
     def on_environment(env: Environment):
-        EnvironmentReport(env).display()
+        _show_status(EnvironmentReport(env), args)
 
-    Context.of(args).run(**locals())
+    context.run(**locals())
 
 
 @command('pkm show package', Arg('dependency'))
@@ -323,7 +386,7 @@ def show_package(args: Namespace):
     def on_environment(env: Environment):
         PackageReport(env, args.dependency).display()
 
-    Context.of(args).run(**locals())
+    context.run(**locals())
 
 
 # noinspection PyUnusedLocal
@@ -338,7 +401,7 @@ def clean_shared(args: Namespace):
     def on_env_zoo(env_zoo: EnvironmentsZoo):
         env_zoo.clean_unused_shared()
 
-    Context.of(args).run(**locals())
+    context.run(**locals())
 
 
 @command('pkm clean dist', Arg(["--all", "-a"], action="store_true"))
@@ -347,28 +410,62 @@ def clean_dist(args: Namespace):
         keep_versions = [project.version] if not args.all else None
         project.directories.clean_dist(keep_versions)
 
-    Context.of(args).run(**locals())
+    context.run(**locals())
+
+
+def prepare_parser() -> ArgumentParser:
+    pkm_parser = create_args_parser(
+        "pkm - python package management for busy developers", globals().values())
+
+    pkm_parser.add_argument('-v', '--verbose', action='store_true', help="run with verbose output")
+    pkm_parser.add_argument('-c', '--context', help="path to the context to run this command under")
+    pkm_parser.add_argument('-g', '--global-context', action='store_true', help="use the global environment context")
+    pkm_parser.add_argument('-nt', '--no-tasks', action='store_true', help="dont run command-attached tasks")
+    pkm_parser.add_argument('-p', '--profile', action='store_true', help="run with cprofiler and print results")
+
+    return pkm_parser
+
+
+@contextmanager
+def profile(args: Namespace):
+    if args.profile:
+        with cProfile.Profile() as pr:
+            yield
+            pstats.Stats(pr).strip_dirs().sort_stats(SortKey.CUMULATIVE).print_stats(20)
+    else:
+        yield
 
 
 def main(args: Optional[List[str]] = None):
+    global context
     args = args or sys.argv[1:]
 
-    def customize_command(cmd: ArgumentParser, _: Command):
-        cmd.add_argument('-v', '--verbose', action='store_true', help="run with verbose output")
-        cmd.add_argument('-c', '--context', help="path to the context to run this command under")
-        cmd.add_argument('-g', '--global-context', action='store_true', help="use the global environment context")
-
-    pkm_parser = create_args_parser(
-        "pkm - python package management for busy developers", globals().values(), customize_command)
+    pkm_parser = prepare_parser()
 
     pargs = pkm_parser.parse_args(args)
-    cli_monitors.listen('verbose' in pargs and pargs.verbose)
-    if 'func' in pargs:
-        pargs.func(pargs)
-    else:
-        pkm_parser.print_help()
+    with profile(pargs):
 
-    Display.print("")
+        cli_monitors.listen('verbose' in pargs and pargs.verbose)
+        context = Context.of(pargs)
+
+        if 'func' in pargs:
+            def with_tasks(p: Project):
+                global tasks
+                tasks = TasksRunner(p)
+                with tasks.run_attached(Command.of(pargs.func), pargs):
+                    pargs.func(pargs)
+
+            def without_tasks():
+                pargs.func(pargs)
+
+            if pargs.no_tasks:
+                without_tasks()
+            else:
+                context.run(on_project=with_tasks, on_missing=without_tasks, silent=True)
+        else:
+            pkm_parser.print_help()
+
+    Display.print("", newline=False)
 
 
 if __name__ == "__main__":
